from django.conf import settings
from django.urls import path, re_path, include
from garpix_user.views import obtain_auth_token, refresh_token_view, logout_view, ChangePasswordView
from rest_framework import routers
from garpix_user.views.user_session_view import UserSessionView
from garpix_user.views.registration_view import registration_view
from garpix_user.views.referral_links_view import ReferralLinkView

from garpix_user.views import (
    EmailConfirmationView, PhoneConfirmationView,
    RestorePasswordView,
    EmailConfirmationLinkView
)

app_name = 'garpix_user'

GARPIX_USER_SETTINGS = getattr(settings, 'GARPIX_USER', dict())

# api routing

router = routers.DefaultRouter()

router.register(r'user_session', UserSessionView, basename='api_user_session')
router.register('', ChangePasswordView, basename='api_change_password')

if GARPIX_USER_SETTINGS.get('USE_EMAIL_CONFIRMATION', False):
    router.register(r'confirm_email', EmailConfirmationView, basename='api_confirm_email')
if GARPIX_USER_SETTINGS.get('USE_PHONE_CONFIRMATION', False):
    router.register(r'confirm_phone', PhoneConfirmationView, basename='api_confirm_phone')
if GARPIX_USER_SETTINGS.get('USE_RESTORE_PASSWORD', False):
    router.register(r'restore_password', RestorePasswordView, basename='api_restore_password')

api_urlpatterns = [
    path('login/', obtain_auth_token, name='api_login'),
    path('refresh/', refresh_token_view, name='api_refresh'),
    path('logout/', logout_view, name='api_logout'),
    path('register/', registration_view, name='api_registration'),
    path(r'', include(router.urls))
]

urlpatterns = [
    path(f'{settings.API_URL}/garpix_user/', include((api_urlpatterns, 'garpix_user'), namespace='garpix_user_api')),
]

if GARPIX_USER_SETTINGS.get('USE_REFERRAL_LINKS', False):
    urlpatterns += [
        re_path(r'invite_link/(?P<hash>.*?)/$', ReferralLinkView.as_view(), name='referral_link'),
        re_path(r'invite_link/(?P<hash>.*?)$', ReferralLinkView.as_view(), name='referral_link')
    ]

if GARPIX_USER_SETTINGS.get('USE_EMAIL_CONFIRMATION', False) and GARPIX_USER_SETTINGS.get('USE_EMAIL_LINK_CONFIRMATION',
                                                                                          True):
    urlpatterns += [
        re_path(r'confirm_email/(?P<model_type>.*?)/(?P<hash>.*?)/$', EmailConfirmationLinkView.as_view(),
                name='email_confirmation_link'),
        re_path(r'confirm_email/(?P<model_type>.*?)/(?P<hash>.*?)$', EmailConfirmationLinkView.as_view(),
                name='email_confirmation_link'),
    ]



from drf_spectacular.utils import extend_schema
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from django.utils.translation import ugettext as _

from garpix_user.serializers import ChangePasswordSerializer
from garpix_user.utils.drf_spectacular import user_session_token_header_parameter


@extend_schema(
    parameters=[
        user_session_token_header_parameter()
    ]
)
class ChangePasswordView(viewsets.ViewSet):

    def get_serializer_class(self):
        return ChangePasswordSerializer
    permission_classes = [IsAuthenticated]

    @extend_schema(summary=_('Change password (for authorized users)'))
    @action(methods=['POST'], detail=False)
    def change_password(self, request, *args, **kwargs):

        user = request.user

        serializer = self.get_serializer_class()(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)

        user.set_password(request.data['new_password'])
        user.save()

        return Response({"result": "success"})



from drf_spectacular.utils import OpenApiParameter
from django.utils.translation import gettext as _
from garpix_user.models import UserSession


def user_session_token_header_parameter(required=False):
    return OpenApiParameter(
        name=UserSession.HEAD_NAME,
        description=_('Unlogged user token'),
        required=required,
        type=str,
        location=OpenApiParameter.HEADER
    )



from django.conf import settings
from rest_framework import serializers

from django.utils.translation import ugettext as _

from garpix_user.utils.repluralize import rupluralize


class PasswordSerializerMixin:

    def _validate_password(self, value):
        GARPIX_USER_SETTINGS = settings.GARPIX_USER

        min_length = GARPIX_USER_SETTINGS.get('MIN_LENGTH_PASSWORD', 8)
        min_digits = GARPIX_USER_SETTINGS.get('MIN_DIGITS_PASSWORD', 2)
        min_chars = GARPIX_USER_SETTINGS.get('MIN_CHARS_PASSWORD', 2)
        min_uppercase = GARPIX_USER_SETTINGS.get('MIN_UPPERCASE_PASSWORD', 1)

        # check for min length
        if len(value) < min_length:
            raise serializers.ValidationError(
                _('Password must be at least {min_length} {characters} long.').format(min_length=min_length,
                                                                                      characters=rupluralize(
                                                                                          min_length,
                                                                                          _('character,characters')))
            )

        # check for min digits number
        if sum(c.isdigit() for c in value) < min_digits:
            raise serializers.ValidationError(
                _('Password must contain at least {min_digits} {digits}.').format(min_digits=min_digits,
                                                                                  digits=rupluralize(min_digits,
                                                                                                     _('digit,digits')))
            )

        # check for min char number
        if sum(c.isalpha() for c in value) < min_chars:
            raise serializers.ValidationError(
                _('Password must contain at least {min_chars} {chars}.').format(min_chars=min_chars,
                                                                                chars=rupluralize(min_chars,
                                                                                                  _('char,chars')))
            )

        # check for uppercase letter
        if sum(c.isupper() for c in value) < min_uppercase:
            raise serializers.ValidationError(
                _('Password must contain at least {min_uppercase} uppercase {letters}.').format(
                    min_uppercase=min_uppercase, letters=rupluralize(min_uppercase, _('letter,letters')))
            )



from django.test import TestCase

from django.contrib.auth import get_user_model


class ApiTestMixin(TestCase):
    def set_init_data(self):
        self.User = get_user_model()
        self.user_data = {
            'username': 'testuser',
            'email': 'test@gmail.com',
            'phone': '+79999999999',
            'password': 'q1W2e3R4t5Y6'
        }
        self.new_user_data = {
            'username': 'testuser1',
            'email': 'test1@gmail.com',
            'phone': '+79999999998',
            'password': 'q1W2e3R4t5Y6'
        }
        self.user = self.User.objects.create_user(**self.user_data)

        self.user.save()

    def user_login(self):
        self.client.force_login(self.user)
        self.client.force_authenticate(self.user)

    def check_login_result(self, response):
        self.assertEqual(response.status_code, 200)
        self.assertIn('access_token', response.json())
        self.assertNotEqual(response.json()['access_token'], '')
        self.assertIn('access_token_expires', response.json())
        self.assertIn('refresh_token', response.json())
        self.assertNotEqual(response.json()['refresh_token'], '')
        self.assertIn('refresh_token_expires', response.json())
        self.assertIn('token_type', response.json())



import json
from django.contrib.auth import authenticate, login, logout
from django.http import JsonResponse
from django.shortcuts import redirect
from django.views.generic.base import RedirectView
from django.views.generic import FormView
from django.http import HttpResponse
from garpix_user.forms import LoginForm


class LogoutView(RedirectView):
    def get_redirect_url(self):
        logout(self.request)
        return self.url


class LoginView(FormView):
    @staticmethod
    def get_form_class(**kwargs):
        return LoginForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context

    def form_valid(self, form):
        request = self.request
        data = form.data
        username = data.get('username')
        password = data.get('password')
        user = authenticate(request, username=username, password=password)
        if user:
            user.set_user_session(request)
        login(request, user)
        if self.request.accepts('text/html'):
            return redirect(request.GET.get('next', '/'))
        return JsonResponse({'success': True})

    def form_invalid(self, form):
        if self.request.accepts('text/html'):
            return self.render_to_response(self.get_context_data(form=form))
        return HttpResponse(json.dumps(form.errors), content_type='application/json', status=400)



from datetime import datetime, timedelta

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.utils.module_loading import import_string


celery_app = import_string(settings.GARPIXCMS_CELERY_SETTINGS)
GARPIX_USER_SETTINGS = getattr(settings, "GARPIX_USER", {})


@celery_app.task()
def delete_unconfirmed_users():
    User = get_user_model()
    filters_data = Q()
    delay_date = datetime.now() - timedelta(days=GARPIX_USER_SETTINGS.get('CONFIRMATION_DELAY', 10))
    if 'email' in User.USERNAME_FIELDS:
        filters_data |= Q(is_email_confirmed=False, date_joined__lt=delay_date)
    if 'phone' in User.USERNAME_FIELDS:
        filters_data |= Q(is_phone_confirmed=False, date_joined__lt=delay_date)

    if filters_data:
        User.objects.filter(filters_data).delete()


celery_app.conf.beat_schedule.update({
    'periodic_task': {
        'task': 'garpix_user.tasks.delete_unconfirmed_users.delete_unconfirmed_users',
        'schedule': 3600,
    }
})
celery_app.conf.timezone = 'UTC'



from .registration_view import RegistrationView  # noqa
from .logout_view import LogoutView, logout_view  # noqa
from .obtain_auth_token import ObtainAuthToken, obtain_auth_token  # noqa
from .refresh_token_view import RefreshTokenView, refresh_token_view  # noqa
from .login_views import LogoutView, LoginView  # noqa

from .email_confirmation_view import EmailConfirmationView, EmailConfirmationLinkView  # noqa
from .phone_confirmation_view import PhoneConfirmationView  # noqa
from .restore_password_view import RestorePasswordView # noqa
from .change_password_view import ChangePasswordView # noqa



from .refresh_token import RefreshToken  # noqa
from .access_token import AccessToken  # noqa
from .user_session import UserSession # noqa
from .refferal import ReferralType, ReferralUserLink  # noqa
from .user import GarpixUser  # noqa



GARPIX_USER_SETTINGS = {
    # base settings
    'USE_REFERRAL_LINKS': False,
    'REFERRAL_REDIRECT_URL': '/',
    # email/phone confirmation
    'USE_EMAIL_CONFIRMATION': False,
    'USE_PHONE_CONFIRMATION': False,
    'USE_PREREGISTRATION_EMAIL_CONFIRMATION': False,
    'USE_PREREGISTRATION_PHONE_CONFIRMATION': False,
    'USE_EMAIL_LINK_CONFIRMATION': False,
    # restore password
    'USE_RESTORE_PASSWORD': True
}

GARPIX_USER_CONFIRMATION_SETTINGS = {
    'USE_EMAIL_CONFIRMATION': True,
    'USE_PHONE_CONFIRMATION': True,
    'USE_PREREGISTRATION_EMAIL_CONFIRMATION': True,
    'USE_PREREGISTRATION_PHONE_CONFIRMATION': True,
    'USE_EMAIL_LINK_CONFIRMATION': False
}
